@@ -1,4 +1,5 @@
 # Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2007 Warp Networks S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -18,9 +19,13 @@ package EBox::Network;
 use strict;
 use warnings;
 
-use base 'EBox::GConfModule';
+use base qw(
+            EBox::GConfModule 
+			EBox::Model::ModelProvider
+			EBox::Model::CompositeProvider
+			EBox::ServiceModule::ServiceInterface
+           ); 
 
-use constant DHCLIENT_CONF_FILE => '/etc/dhcp3/dhclient.conf';
 # Interfaces list which will be ignored
 use constant ALLIFACES => qw(sit tun tap lo irda eth wlan vlan);
 use constant IGNOREIFACES => qw(sit tun tap lo irda);
@@ -37,17 +42,20 @@ use EBox::Exceptions::DataInUse;
 use EBox::Exceptions::Internal;
 use EBox::Exceptions::External;
 use Error qw(:try);
+use EBox::Summary::Composite;
 use EBox::Summary::Module;
-use EBox::Summary::Value;
 use EBox::Summary::Section;
+use EBox::Summary::Value;
 use EBox::Menu::Item;
 use EBox::Menu::Folder;
 use EBox::Sudo qw( :all );
 use EBox::Gettext;
 #use EBox::LogAdmin qw( :all );
 use File::Basename;
-use EBox::Network::Model::GatewayDataTable;
-use EBox::Network::Model::MultiGwRulesDataTable;
+
+
+use EBox::Network::Report::ByteRate;
+
 
 sub _create
 {
@@ -57,21 +65,83 @@ sub _create
 					domain => 'ebox-network',
 					@_);
 	$self->{'actions'} = {};
-	$self->{'gatewayModel'} = new EBox::Network::Model::GatewayDataTable(
-					'gconfmodule' => $self,
-					'directory' => 'gatewaytable',
-					);
-	
-	$self->{'multigwrulesModel'} = 
-				new EBox::Network::Model::MultiGwRulesDataTable(
-					'gconfmodule' => $self,
-					'directory' => 'multigwrulestable',
-					);
+
 	bless($self, $class);
 	
 	return $self;
 }
 
+# Method: actions
+#
+# 	Override EBox::ServiceModule::ServiceInterface::actions
+#
+sub actions
+{
+	return [ 
+	{
+		'action' => __('Add default routers to the default table'),
+		'reason' => __('This is needed to work with a multigateway ' .
+					'configuration. Note that to list the default routes you ' .
+					'must execute: ') . ' ip route ls table default ',
+		'module' => 'network'
+	},
+	{	
+		'action' => __('Enable eBox DHCP hook'),
+		'reason' => __('It will take care of adding the default route' .
+				' given by a DHCP server to the default route table. '),
+		'module' => 'network'
+	}
+	];
+}
+
+#  Method: serviceModuleName
+#
+#   Override EBox::ServiceModule::ServiceInterface::servivceModuleName
+#
+sub serviceModuleName
+{
+	return 'network';
+}
+
+sub modelClasses
+{
+  return [
+	  { 
+	   class      => 'EBox::Network::Model::GatewayTable',
+	   parameters => [
+                          directory => 'gatewaytable',
+                         ],
+	  },
+	  {
+           class     => 'EBox::Network::Model::MultiGwRulesDataTable',
+           parameters => [
+                          directory => 'multigwrulestable',
+                         ],
+          },
+          {
+           class      => 'EBox::Network::Model::ByteRateEnableForm',
+           parameters => [
+                          enableTitle  => __('Activate traffic rate monitor'),
+                          domain       => 'ebox-network',
+                          modelDomain  => 'Network',
+                         ],
+          },
+	  'EBox::Network::Model::ByteRateSettings',
+	  'EBox::Network::Model::ByteRateGraph',
+	  'EBox::Network::Model::ByteRateGraphControl',
+	 ];
+}
+
+
+
+sub compositeClasses
+{
+  return [
+	  'EBox::Network::Composite::ByteRate',
+	 ];
+
+}
+
 # Method: IPAddressExists
 #
 # 	Returns true if the given IP address belongs to a statically configured
@@ -573,7 +643,7 @@ sub setViface # (real, virtual, address, netmask)
 			my $ex = shift;
 			settextdomain('ebox');
 			throw $ex;
-		}
+		};
 	}
 	settextdomain('ebox');
 
@@ -955,7 +1025,7 @@ sub _checkStatic # (iface, force)
 
 	foreach my $vif (@{$self->vifaceNames($iface)}) {
 		foreach my $mod (@mods) {
-			my ($tmp, $viface) = $self->_viface2array($iface);
+			my ($tmp, $viface) = $self->_viface2array($vif);
 			if ($mod->vifaceDelete($iface, $viface)) {
 				if ($force) {
 					$mod->freeViface($iface, $viface);
@@ -1651,22 +1721,10 @@ sub _generateRoutes
 		if (route_is_up($net, $router)) {
 			root("/sbin/ip route del $net via $router");
 		}
-		root("/sbin/ip route add $net via $router table default || true");
+		root("/sbin/ip route add $net via $router table main || true");
 	}
 }
 
-sub _generateDHCPClientConf
-{
-	my $self = shift;
-	
-	my @params = ('script' => 
-		EBox::Config::libexec . "../ebox-network/dhclient-script");
-	
-	$self->writeConfFile(DHCLIENT_CONF_FILE,
-				'network/dhclient.conf.mas',
-				\@params);
-}
-
 sub _multigwRoutes
 {
 	my $self = shift;
@@ -1696,9 +1754,8 @@ sub _multigwRoutes
 	# We modify the dhclient script behaviour to add the
 	# default route where we need it.
 
-	$self->_generateDHCPClientConf();
 	
-	root(EBox::Config::libexec . "../ebox-network/ebox-flush-fwmarks");
+	root(EBox::Config::share() . "ebox-network/ebox-flush-fwmarks");
 	my $marks = $self->marksForRouters();
 	my $routers = $self->gatewaysWithMac();
 	for my $router (@{$routers}) {
@@ -1749,6 +1806,12 @@ sub _multigwRoutes
 		root("/sbin/iptables -t mangle -A PREROUTING "
 			."-j CONNMARK --save-mark");
 	} catch EBox::Exceptions::Internal with {};
+
+        try {
+                root("/sbin/iptables -t mangle -I OUTPUT "
+                        ."-j CONNMARK --restore-mark");
+        } catch EBox::Exceptions::Internal with {};
+
 }
 
 # Method: _regenConfig
@@ -1831,7 +1894,7 @@ sub _regenConfig
 		}
 	}
 	foreach (@ifups) {
-		root("/sbin/ifup --force -i $file $_");
+		root(EBox::Config::pkgdata() . "ebox-unblock-exec /sbin/ifup --force -i $file $_");
 		unless ($self->isReadOnly()) {
 			$self->_unsetChanged($_);
 		}
@@ -1853,6 +1916,9 @@ sub _regenConfig
 	$self->_generateRoutes();
 	$self->_multigwRoutes();
 	$self->_cleanupVlanIfaces();
+
+	# regenerate config for the bit rate report
+	EBox::Network::Report::ByteRate->_regenConfig();
 }
 
 sub stopService
@@ -1868,6 +1934,8 @@ sub stopService
 			root("/sbin/ifdown --force -i $file $if");
 		} catch EBox::Exceptions::Internal with {};
 	}
+
+	EBox::Network::Report::ByteRate->stopService();
 }
 
 #internal use functions
@@ -2236,6 +2304,7 @@ sub resolv # (host)
 sub summary
 {
 	my $self = shift;
+        my $composite = new EBox::Summary::Composite();
 	my $item = new EBox::Summary::Module(__("Network interfaces"));
 	my $ifaces = $self->ifacesWithRemoved;
 	my $linkstatus = {};
@@ -2284,7 +2353,13 @@ sub summary
 				(__("IP address"), $ip));
 		}
 	}
-	return $item;
+	$composite->add($item);
+        my $monSummary = EBox::Network::Report::ByteRate->summary();
+        if ( defined($monSummary) ) {
+            $composite->add($monSummary);
+        }
+
+        return $composite;
 }
 
 # Method: menu 
@@ -2308,11 +2383,16 @@ sub menu
 	$folder->add(new EBox::Menu::Item('url' => 'Network/Diag',
 					  'text' => __('Diagnosis')));
 	$folder->add(new EBox::Menu::Item('url' => 
-						'Network/View/GatewayDataTable',
+						'Network/View/GatewayTable',
 					  'text' => __('Gateways')));
 	$folder->add(new EBox::Menu::Item('url' => 
 						'Network/View/MultiGwRulesDataTable',
 					  'text' => __('Balance traffic')));
+	$folder->add(new EBox::Menu::Item('url' => 
+						'Network/Composite/ByteRate',
+					  'text' => __('Traffic rate monitor')));
+
+
 
 	$root->add($folder);
 }
@@ -2325,11 +2405,10 @@ sub menu
 #
 # 	GatewayTableModel
 #
+#  XXX delete and replace calls with direct call to the 'model' method
 sub gatewayModel {
-	
 	my $self = shift;
-
-	return $self->{'gatewayModel'};
+	return $self->model('GatewayTable');
 }
 
 # Method: multigwrulesModel
@@ -2340,11 +2419,12 @@ sub gatewayModel {
 #
 # 	MultiGwRuleTableModel
 #
+#  XXX delete and replace calls with direct call to the 'model' method
 sub multigwrulesModel {
 	
 	my $self = shift;
 
-	return $self->{'multigwrulesModel'};
+	return $self->model('MultiGwRulesDataTable');
 }
 
 
